﻿using System.Text.Json.Serialization;
using FFmpeg.AutoGen;

namespace FNZCM2.Abstractions.Models.Metadata;

public class FileMetaEntry
{
    [JsonPropertyName("i")] public required long FileId { get; init; }

    [JsonPropertyName("t")] public required FileType FileType { get; init; }

    [JsonPropertyName("l")] public required int? DurationMs { get; init; }

    [JsonPropertyName("b")] public required int? BitRate { get; init; }
    [JsonPropertyName("p")] public required IReadOnlyList<AVCodecID> Codecs { get; init; }
    [JsonPropertyName("p")] public required PictureInfo? Picture { get; init; }
    [JsonPropertyName("a")] public required AudioInfo? Audio { get; init; }
    [JsonPropertyName("m")] public required MetaInfo Meta { get; init; }

    public override string ToString()
    {
        return $"{FileType} {string.Join("|", Codecs.Select(p => p.ToString().Substring("AV_CODEC_ID_".Length)))} {Picture} {Audio}";
    }
}

public enum FileType
{
    Unknown = 0,
    Audio,
    Video,
    Picture,
    Text,
    Doc,
}
